install.packages("ggplot2")
install.packages("devtools")
install.packages("purrr")
install.packages("rmda")
install.packages("Hmisc")
install.packages("rms")
install.packages("BioPET")
intall.packages("pROC")
intall.packages("gridExtra")
library(gridExtra)
library(ggplot2)
library(devtools)
library(purrr)
library(rmda)
library(Hmisc)
library(rms)


#load (simulated) data for risk of  cancer
data(dcaData)
head(dcaData)
#prevalence of cancer in the dataset is 12%
mean(dcaData$Cancer)

## FITTING RISK MODELS USING LOGISTIC REGRESSION, ASSESSING CALIBRATION, OPTIMISM-CORRECTED PERFORMANCE
##
# use logistic regression to estimate risk function that uses Age, Sex, and two Markers
f <- lrm(Cancer ~ Age + Female + Smokes + Marker1 + Marker2, data = dcaData)
print(f)
# C stands for "c-index"; it is the apparent AUC
# Dxy is Somner's D = 2(AUC-0.5) ; AUC = 0.5*D + 0.5


# assess Mean Calibration
# first, get predicted risks for individuals in the dataset
fitted.risks <- predict(f,dcaData,type="fitted")
mean(fitted.risks)

# Assess Logistic Calibration
# model fit must return the expanded design matrix
f <- lrm(Cancer ~ Age + Female + Smokes + Marker1 + Marker2, data = dcaData, x=TRUE, y=TRUE)
# set the seed for bootstrapping
set.seed(1796)
validate(f, B=400)
# look at the index.corrected column for optimism-corrected estimates
# logistic intercept is 0 in the index.orig column; optimism-corrected is -0.0438
# logistic slope is 1 in the index.org column;      optimism-corrected is  0.9474 
#    - indicating that high risks are overestimated and low risks are underestimated

# Assess Moderate Calibration
# Calibration curve
# use function with the RMS package
# m tells average number of people in each group; dataset has 500 people, so m=50 for deciles
# val. prob computes various metrics and also plots calibration curve when pl=TRUE
val.prob(predict(f,dcaData,type="fitted"), dcaData$Cancer, m=50, statloc=FALSE, pl=TRUE)
# another way to get a calibration curve
f.cal <- calibrate(f, method = "boot", B=400)
plot(f.cal)

##  INCREMENTAL VALUE
# Suppose that Marker 1 is an established biomarker and Marker 2 is a potentially useful new biomarker
set.seed(9876)
# DecisionCurve calculations for predicted risks based on  age, sex, smoking status, and Marker1
# set bootstraps = 50 here to reduce computation time.
baseline.model <- decision_curve(Cancer~Age + Female + Smokes + Marker1, #fitting a logistic model
                                data = dcaData,
                                study.design = "cohort",
                                bootstraps = 100)

# plot the Decision curve for baseline model
plot_decision_curve(baseline.model,  curve.names = "baseline model")
plot_decision_curve(baseline.model,  curve.names = "baseline model", xlim=c(0, .20), ylim=c(0, 1))
# we can also get a table summarizing results
summary(baseline.model)

# switch to Net Benefit rather than standardized Net Benefit
plot_decision_curve(baseline.model, standardize=F, curve.names = "baseline model", xlim=c(0, 0.15), ylim=c(0, 0.15))

# Examine the potential for the new biomarker to improve Net Benefit
full.model <- decision_curve(Cancer~Age + Female + Smokes + Marker1 + Marker2, data = dcaData, bootstraps = 50)
# since we want to plot more than one curve, we pass a list of 'DecisionCurve' objects to the plot
plot_decision_curve( list(baseline.model, full.model),  curve.names = c("Baseline model", "Full model"), xlim=c(0, .20), ylim=c(0,1))


# If the standard is biopsy, then "opt-out" decision curve is more useful
baseline.model.optout <- decision_curve(Cancer~Age + Female + Smokes + Marker1, #fitting a logistic model
                                        data = dcaData,
                                        study.design = "cohort",
                                        policy = "opt-out",
                                        bootstraps = 50)
plot_decision_curve(baseline.model.optout, curve.names = "baseline model", xlim=c(0, .12), ylim=c(0, 0.6))


# in the Opt-Out formulation, examine the evidence for the new marker
full.model.optout <- decision_curve(Cancer~Age + Female + Smokes + Marker1 + Marker2, policy="opt-out", data = dcaData, bootstraps = 50)
plot_decision_curve( list(baseline.model.optout, full.model.optout), curve.names = c("baseline", "full"), xlim=c(0, .12), legend.position="topleft")

# Plot ROC alternative for full model that includes biomarkers
plot_roc_components(full.model,  xlim = c(0, 0.20),col = c("black", "red"))

# Clinical Impact Plot for full model that includes biomarkers
plot_clinical_impact(full.model, xlim = c(0, .20), col = c("black", "blue"))


 

## EVALUATE TWO RISK MODELS FOR PROGNOSTIC ENRICHMENT OF CLINICAL TRIAL
library(BioPET)
library(pROC)
library(gridExtra)
rm(list=ls())
enrichment.baseline.model<-enrichment_analysis(formula=Cancer~Age+Smokes+Marker1, data=dcaData, 
                                               cost.screening=50, cost.keeping=500, reduction.under.treatment=0.3)
plot_enrichment_summaries(enrichment.baseline.model)
enrichment.full.model<-enrichment_analysis(formula=Cancer~Age+Smokes+Marker1+Marker2, data=dcaData, 
                                           cost.screening=50, cost.keeping=500, reduction.under.treatment=0.3)
plot_enrichment_summaries(enrichment.full.model)

